﻿Imports System.IO
Imports System.ComponentModel
Imports System.Runtime.InteropServices

''' <summary>
''' フォルダツリービュー。System.Windows.Forms.TreeViewを継承
''' </summary>
''' <remarks></remarks>
Public Class FolderTreeView
    Inherits System.Windows.Forms.TreeView

#Region "定義・メンバー変数"
    Private Const CtrlMask As Integer = 8 ' キーボード情報取得用
    Private Const ShiftMask As Integer = 4 ' キーボード情報取得用

    Private components As System.ComponentModel.IContainer

    ' プロパティ用変数
    Private prop_showMyPicthre As Boolean
    Private prop_showRemovable As Boolean
    Private prop_showNetwork As Boolean
    Private prop_showFloppy As Boolean
    Private prop_showMyDocument As Boolean

    Public Enum SHGFI ' シェルAPI呼び出し用
        SHGFI_ICON = &H100
        SHGFI_DISPLAYNAME = &H200
        SHGFI_TYPENAME = &H400
        SHGFI_ATTRIBUTES = &H800
        SHGFI_ICONLOCATION = &H1000
        SHGFI_EXETYPE = &H2000
        SHGFI_SYSICONINDEX = &H4000
        SHGFI_LINKOVERLAY = &H8000
        SHGFI_SELECTED = &H10000
        SHGFI_ATTR_SPECIFIED = &H20000
        SHGFI_LARGEICON = &H0
        SHGFI_SMALLICON = &H1
        SHGFI_OPENICON = &H2
        SHGFI_SHELLICONSIZE = &H4
        SHGFI_PIDL = &H8
        SHGFI_USEFILEATTRIBUTES = &H10
        SHGFI_ADDOVERLAYS = &H20
        SHGFI_OVERLAYINDEX = &H40
    End Enum

    Public Structure SHFILEINFO ' シェルAPI呼び出し用
        Public hIcon As IntPtr
        Public iIcon As Integer
        Public dwAttributes As UInt32
        <MarshalAs(UnmanagedType.ByValTStr, SizeConst:=260)> _
        Public szDisplayName As String
        <MarshalAs(UnmanagedType.ByValTStr, SizeConst:=80)> _
        Public szTypeName As String
    End Structure

    Private m_nodeHighlight As TreeNode = Nothing ' ドラッグ＆ドロップ用
    Private m_tnDrag As TreeNode = Nothing ' ドラッグ使用としているツリー
    Private m_bDragNow As Boolean = False ' ドラッグ中かどうか
#End Region

#Region "システム関数"
    ''' <summary>
    ''' FolderTreeViewクラスのコンストラクタ
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub New()
        prop_showMyPicthre = True
        prop_showRemovable = True
        prop_showNetwork = False
        prop_showFloppy = False

        Me.SetStyle(ControlStyles.Opaque, True)

        InitializeComponent()
    End Sub

    ''' <summary>
    ''' 初期化ルーチン
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub InitializeComponent()
        Me.SuspendLayout()
        '
        'FolderTreeView
        '
        Me.AllowDrop = True
        Me.Font = New System.Drawing.Font("MS UI Gothic", 12.0!, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, CType(128, Byte))
        Me.LineColor = System.Drawing.Color.Black
        Me.ResumeLayout(False)

    End Sub

    ''' <summary>
    ''' ツリーが展開される時に呼び出される
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub FolderTreeView_AfterExpand(ByVal sender As System.Object, ByVal e As System.Windows.Forms.TreeViewEventArgs) Handles MyBase.AfterExpand
        If DesignMode Then
            Return
        End If
        Me.BeginUpdate()
        e.Node.Nodes.Clear()
        AddTree(e.Node.Name, e.Node.Nodes, 1)
        Me.EndUpdate()
    End Sub

    ''' <summary>
    ''' クリックされた時に呼び出される
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub FolderTreeView_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Click
        Dim mouseEvent As MouseEventArgs = CType(e, MouseEventArgs)
        If Not Me.ContextMenuStrip Is Nothing Then
            Dim thi As TreeViewHitTestInfo
            thi = Me.HitTest(mouseEvent.Location)
            If mouseEvent.Button = Windows.Forms.MouseButtons.Right Then ' 右クリックでも選択状態にする

                If thi.Node IsNot Nothing Then
                    Me.SelectedNode = thi.Node
                End If
            End If
        End If
    End Sub

    ''' <summary>
    ''' ラベルの編集が終わる時に呼び出される
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub FolderTreeView_AfterLabelEdit(ByVal sender As System.Object, ByVal e As System.Windows.Forms.NodeLabelEditEventArgs) Handles MyBase.AfterLabelEdit
        System.GC.Collect() ' GCを開放して、オープンしてるフォルダ名の変更をできるようにする
        System.GC.WaitForPendingFinalizers()

        Dim newname As String
        newname = e.Node.Parent.Name + "\" + e.Label

        Try
            System.IO.Directory.Move(e.Node.Name, newname)
            e.Node.Name = newname
            System.GC.Collect()
            System.GC.WaitForPendingFinalizers()
            Me.SelectedNode = e.Node
        Catch ex As Exception
            e.CancelEdit = True ' 変更に失敗するならラベルも元に戻す
        End Try
    End Sub

    ''' <summary>
    ''' ラベルの編集が開始される時に呼び出される
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub FolderTreeView_BeforeLabelEdit(ByVal sender As System.Object, ByVal e As System.Windows.Forms.NodeLabelEditEventArgs) Handles MyBase.BeforeLabelEdit
        If e.Node.Level < 1 Then ' ルートの名前は変えられないようにする
            e.CancelEdit = True
        End If
    End Sub

    ''' <summary>
    ''' ドラッグ＆ドロップ処理。ファイル・フォルダがドラッグされてきたらカーソルやハイライトを変更する
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub FolderTreeView_DragOver(ByVal sender As System.Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles MyBase.DragOver
        Dim p As Point = MyBase.PointToClient(New Point(e.X, e.Y))
        Dim tn As TreeNode = MyBase.GetNodeAt(p)
        If tn IsNot Nothing Then
            Dim filePathName As String() = e.Data.GetData(DataFormats.FileDrop, False)
            If (filePathName.Length > 0) Then
                Dim strDragFolderPath As String = ""
                Try
                    strDragFolderPath = Path.GetDirectoryName(filePathName(0))
                Catch ex As Exception

                End Try
                If m_nodeHighlight IsNot tn Or strDragFolderPath IsNot tn.Name Then
                    If m_nodeHighlight IsNot Nothing Then
                        m_nodeHighlight.BackColor = Me.BackColor
                        m_nodeHighlight.ForeColor = Me.ForeColor
                    End If
                    tn.BackColor = SystemColors.Highlight
                    tn.ForeColor = SystemColors.HighlightText
                    m_nodeHighlight = tn
                    If e.Data.GetDataPresent(DataFormats.FileDrop) Then
                        Dim bCurFolder As Boolean = True

                        For Each strFile As String In filePathName
                            If strFile.CompareTo(tn.Name) = 0 Then

                            ElseIf Not Path.GetDirectoryName(strFile).CompareTo(tn.Name) = 0 Then
                                bCurFolder = False
                            End If
                        Next

                        If bCurFolder Then
                            e.Effect = DragDropEffects.None
                            Return
                        End If

                        If (e.KeyState And CtrlMask) = CtrlMask And (e.AllowedEffect And DragDropEffects.Copy) = DragDropEffects.Copy Then
                            e.Effect = DragDropEffects.Copy
                        ElseIf ((e.KeyState And ShiftMask) = ShiftMask And (e.AllowedEffect And DragDropEffects.Move) = DragDropEffects.Move) Then
                            e.Effect = DragDropEffects.Move
                        Else
                            e.Effect = DragDropEffects.Copy
                        End If
                    Else
                        e.Effect = DragDropEffects.None
                    End If
                Else
                    e.Effect = DragDropEffects.None
                End If
                End If
            Else
                If m_nodeHighlight IsNot Nothing Then
                    m_nodeHighlight.BackColor = Me.BackColor
                    m_nodeHighlight.ForeColor = Me.ForeColor
                End If
                e.Effect = DragDropEffects.None
            End If
    End Sub

    ''' <summary>
    ''' ドラッグ＆ドロップ処理。ファイル・フォルダがドラッグされてきたらカーソルやハイライトを変更する
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub FolderTreeView_DragEnter(ByVal sender As System.Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles MyBase.DragEnter
        If e.Data.GetDataPresent(DataFormats.FileDrop) Then
            Dim filePathName As String() = e.Data.GetData(DataFormats.FileDrop, False)
            Dim p As Point = Me.PointToClient(New Point(e.X, e.Y))
            Dim tn As TreeNode = Me.GetNodeAt(p)

            Dim bCurFolder As Boolean = True
            If tn IsNot Nothing Then
                For Each strFile As String In filePathName
                    If strFile.CompareTo(tn.Name) = 0 Then

                    ElseIf Not Path.GetDirectoryName(strFile).CompareTo(tn.Name) = 0 Then
                        bCurFolder = False
                    End If
                Next
            End If
            If bCurFolder Then
                e.Effect = DragDropEffects.None
                Return
            End If

            If ((e.KeyState And CtrlMask) = CtrlMask) And ((e.AllowedEffect And DragDropEffects.Copy) = DragDropEffects.Copy) Then
                e.Effect = DragDropEffects.Copy
            ElseIf ((e.KeyState And ShiftMask) = ShiftMask) And ((e.AllowedEffect And DragDropEffects.Move) = DragDropEffects.Copy) Then
                e.Effect = DragDropEffects.Move
            Else
                e.Effect = DragDropEffects.Copy
            End If
        Else
            e.Effect = DragDropEffects.None
        End If
    End Sub

    ''' <summary>
    ''' ドラッグ＆ドロップ処理。アイテムの色を元に戻す
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub FolderTreeView_DragLeave(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.DragLeave
        If m_nodeHighlight IsNot Nothing Then
            m_nodeHighlight.BackColor = Me.BackColor
            m_nodeHighlight.ForeColor = Me.ForeColor
        End If
    End Sub

    ''' <summary>
    ''' ドラッグ＆ドロップ処理。フォルダ・ファイルのコピーを行う
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub FolderTreeView_DragDrop(ByVal sender As System.Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles MyBase.DragDrop
        Dim bCopyFlag As Boolean = True
        If e.Effect = DragDropEffects.Move Then
            bCopyFlag = False
        End If

        Dim filePathName As String() = e.Data.GetData(DataFormats.FileDrop, False)
        Dim p As Point = Me.PointToClient(New Point(e.X, e.Y))
        Dim tn As TreeNode = Me.GetNodeAt(p)

        If tn IsNot Nothing Then
            For Each strFile As String In filePathName
                If strFile.CompareTo(tn.Name) = 0 Then

                ElseIf Not Path.GetDirectoryName(strFile).CompareTo(tn.Name) = 0 Then
                    If e.Effect = DragDropEffects.Move Then
                        CopyFolders(strFile, tn.Name, False, True)
                    ElseIf e.Effect = DragDropEffects.Copy Then
                        CopyFolders(strFile, tn.Name, False, False)
                    End If
                End If
            Next
        End If

        If m_nodeHighlight IsNot Nothing Then
            m_nodeHighlight.BackColor = Me.BackColor
            m_nodeHighlight.ForeColor = Me.ForeColor
        End If
    End Sub

    ''' <summary>
    ''' ショートカットキー処理
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub GraphicCollectorForm_KeyDown(ByVal sender As System.Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles MyBase.KeyDown
        Select Case e.KeyCode
            Case Keys.F5 ' F5：リロード
                ReloadFolders()

            Case Keys.F2 ' F2：選択されているフォルダのリネーム
                RenameFolder()

            Case Keys.D ' Ctrl+D：選択されているフォルダの削除
                If e.Control Then
                    If DeleteCurrentFolder() = False Then
                        MessageBox.Show("フォルダを削除できません。", My.Application.Info.Title, MessageBoxButtons.OK)
                    End If
                End If

            Case Keys.Delete ' Delete：選択されているフォルダの削除
                If DeleteCurrentFolder() = False Then
                    MessageBox.Show("フォルダを削除できません。", My.Application.Info.Title, MessageBoxButtons.OK)
                End If

            Case Keys.C ' Ctrl+C：コピー
                If e.Control Then
                    ClipCopyFolder()
                End If

            Case Keys.V ' Ctrl+V：貼り付け
                If e.Control Then
                    PasteFolder()
                End If

            Case Else

        End Select
    End Sub

    ''' <summary>
    ''' マウス押下。ツリーからのドラッグ＆ドロップ処理のための準備を行う
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub FolderTreeView_MouseDown(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseDown
        m_tnDrag = Nothing
        If e.Button = Windows.Forms.MouseButtons.Left Then
            Dim tn As TreeNode = Me.GetNodeAt(e.X, e.Y)
            If tn IsNot Nothing Then
                ' ルートツリーは対象としない
                If tn.Parent IsNot Nothing Then
                    m_tnDrag = tn
                End If
            End If
        End If
    End Sub

    ''' <summary>
    ''' マウス移動。ツリーからのドラッグ＆ドロップ処理を行う
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub FolderTreeView_MouseMove(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseMove
        Dim bFlag As Boolean = False
        If m_tnDrag IsNot Nothing Then
            Dim tn As TreeNode = Me.GetNodeAt(e.X, e.Y)
            If tn Is Nothing Then
                bFlag = True
            ElseIf Not TreeNode.ReferenceEquals(tn, m_tnDrag) Then
                bFlag = True
            End If

            If bFlag Then
                m_bDragNow = True
                Dim paths As String() = {m_tnDrag.Name}
                Dim data As New DataObject(DataFormats.FileDrop, paths)
                Dim dde As DragDropEffects

                dde = Me.DoDragDrop(data, DragDropEffects.All)

                Dim parent As TreeNode = m_tnDrag.Parent
                If parent IsNot Nothing Then
                    parent.Nodes.Clear()
                    AddTree(parent.Name, parent.Nodes, 1)
                    parent.Expand()
                End If

                m_bDragNow = False
                m_tnDrag = Nothing
            End If
        End If
    End Sub

    ''' <summary>
    ''' マウスが離れた場合。ドラッグ＆ドロップ処理が行われていなければ初期化する
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub FolderTreeView_MouseLeave(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.MouseLeave
        If Not m_bDragNow Then
            m_tnDrag = Nothing
        End If
    End Sub

    ''' <summary>
    ''' マウスボタンが離された場合。ドラッグ＆ドロップ処理が行われていなければ初期化する
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub FolderTreeView_MouseUp(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseUp
        If Not m_bDragNow Then
            m_tnDrag = Nothing
        End If
    End Sub

#End Region

#Region "ユーザー関数"
    ''' <summary>
    '''  SHGetFileInfo：シェルAPI。アイコン読み込みの為に使用
    ''' </summary>
    ''' <param name="pszPath"></param>
    ''' <param name="dwFileAttribs"></param>
    ''' <param name="psfi"></param>
    ''' <param name="cbFileInfo"></param>
    ''' <param name="uFlags"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <DllImport("shell32.dll", CharSet:=CharSet.Unicode)> _
        Public Shared Function SHGetFileInfo(ByVal pszPath As String, ByVal dwFileAttribs As UInt32, _
        ByRef psfi As SHFILEINFO, ByVal cbFileInfo As UInt32, ByVal uFlags As SHGFI) As IntPtr
    End Function


    ''' <summary>
    ''' ツリービューの初期化を行う
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub InitView()
        If DesignMode Then
            Return
        End If

        Me.ForeColor = Color.Blue

        InitTree()

        Dim nodes() As TreeNode
        nodes = Me.Nodes.Find(Environment.GetFolderPath(Environment.SpecialFolder.MyPictures), True)
        If nodes.Length > 0 Then
            Me.SelectedNode = nodes(0)
        End If

    End Sub

    ''' <summary>
    ''' ツリーの構築を行う
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub InitTree()
        Dim node As TreeNode
        Dim sLabel As String
        Dim sName As String

        Me.Nodes.Clear()
        If Me.ImageList IsNot Nothing Then
            Me.ImageList.Dispose()
        End If

        Dim imgList As ImageList = New ImageList
        imgList.ImageSize = New Size(16, 16)
        imgList.ColorDepth = ColorDepth.Depth32Bit

        Try
            Dim sf As SHFILEINFO = New SHFILEINFO
            SHGetFileInfo(Environment.SystemDirectory, 0, sf, Marshal.SizeOf(sf), SHGFI.SHGFI_SMALLICON + SHGFI.SHGFI_ICON)
            imgList.Images.Add(Icon.FromHandle(sf.hIcon))
            SHGetFileInfo(Environment.SystemDirectory, 0, sf, Marshal.SizeOf(sf), SHGFI.SHGFI_SMALLICON + SHGFI.SHGFI_ICON + SHGFI.SHGFI_OPENICON)
            imgList.Images.Add(Icon.FromHandle(sf.hIcon))
            Me.ImageList = imgList
            Me.SelectedImageIndex = 1
        Catch ex As Exception
        End Try

        If prop_showMyPicthre Then
            sName = Environment.GetFolderPath(Environment.SpecialFolder.MyPictures)
            If sName.Length = 0 Then

            ElseIf Directory.Exists(sName) Then
                Try
                    Dim sf As SHFILEINFO = New SHFILEINFO
                    Dim imgid As Integer
                    imgid = Me.ImageList.Images.Count
                    SHGetFileInfo(sName, 0, sf, Marshal.SizeOf(sf), SHGFI.SHGFI_SMALLICON + SHGFI.SHGFI_ICON)
                    Me.ImageList.Images.Add(Icon.FromHandle(sf.hIcon))
                    node = New TreeNode()
                    node.Text = "マイ ピクチャ"
                    node.Name = sName
                    node.ImageIndex = imgid
                    'imgid = Me.ImageList.Images.Count
                    'SHGetFileInfo(sName, 0, sf, Marshal.SizeOf(sf), SHGFI.SHGFI_SMALLICON + SHGFI.SHGFI_ICON + SHGFI.SHGFI_OPENICON)
                    'Me.ImageList.Images.Add(Icon.FromHandle(sf.hIcon))
                    node.SelectedImageIndex = imgid
                    AddTree(sName, node.Nodes, 1)
                    Me.Nodes.Add(node)
                Catch ex As Exception

                End Try
            End If
        End If

        If prop_showMyDocument Then
            sName = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments)
            If sName.Length = 0 Then

            ElseIf Directory.Exists(sName) Then
                Try
                    Dim sf As SHFILEINFO = New SHFILEINFO
                    Dim imgid As Integer
                    imgid = Me.ImageList.Images.Count
                    SHGetFileInfo(sName, 0, sf, Marshal.SizeOf(sf), SHGFI.SHGFI_SMALLICON + SHGFI.SHGFI_ICON)
                    Me.ImageList.Images.Add(Icon.FromHandle(sf.hIcon))
                    node = New TreeNode()
                    node.Text = "マイ ドキュメント"
                    node.Name = sName
                    node.ImageIndex = imgid
                    'imgid = Me.ImageList.Images.Count
                    'SHGetFileInfo(sName, 0, sf, Marshal.SizeOf(sf), SHGFI.SHGFI_SMALLICON + SHGFI.SHGFI_ICON + SHGFI.SHGFI_OPENICON)
                    'Me.ImageList.Images.Add(Icon.FromHandle(sf.hIcon))
                    node.SelectedImageIndex = imgid
                    AddTree(sName, node.Nodes, 1)
                    Me.Nodes.Add(node)
                Catch ex As Exception

                End Try
            End If
        End If

        Dim di As DriveInfo()
        di = System.IO.DriveInfo.GetDrives()
        For Each d As DriveInfo In di
            Try
                If Not prop_showFloppy And (d.Name = "A:\" Or d.Name = "B:\") Then
                    Continue For
                End If

                If (Not prop_showNetwork) And (d.DriveType = DriveType.Network) Then
                    Continue For
                End If

                If (Not prop_showRemovable) And (d.DriveType = DriveType.CDRom Or d.DriveType = DriveType.Removable Or d.DriveType = DriveType.Unknown) Then
                    Continue For
                End If


                Try
                    Dim sf As SHFILEINFO = New SHFILEINFO
                    Dim imgid As Integer
                    imgid = Me.ImageList.Images.Count
                    SHGetFileInfo(d.Name, 0, sf, Marshal.SizeOf(sf), SHGFI.SHGFI_SMALLICON + SHGFI.SHGFI_ICON)
                    Me.ImageList.Images.Add(Icon.FromHandle(sf.hIcon))
                    node = New TreeNode()
                    node.Name = d.Name
                    node.ImageIndex = imgid
                    'imgid = Me.ImageList.Images.Count
                    'SHGetFileInfo(d.Name, 0, sf, Marshal.SizeOf(sf), SHGFI.SHGFI_SMALLICON + SHGFI.SHGFI_ICON + SHGFI.SHGFI_OPENICON)
                    'Me.ImageList.Images.Add(Icon.FromHandle(sf.hIcon))
                    node.SelectedImageIndex = imgid
                    sLabel = ""
                    If d.IsReady Then
                        sLabel = d.VolumeLabel
                    End If
                    If sLabel.Length < 1 Then
                        node.Text = d.Name
                    Else
                        node.Text = String.Format("{0} ({1})", sLabel, d.Name.Substring(0, 2))
                    End If
                    AddTree(d.Name, node.Nodes, 1)
                    Me.Nodes.Add(node)
                Catch ex As Exception

                End Try
            Catch ex As Exception

            End Try
        Next
    End Sub

    ''' <summary>
    ''' サブフォルダのツリーを構築する。
    ''' </summary>
    ''' <param name="sDir">検索するフォルダ</param>
    ''' <param name="nodes">設定するTreeNodeCollection</param>
    ''' <param name="level">levelで指定した階層まで作成する</param>
    ''' <remarks></remarks>
    Private Sub AddTree(ByVal sDir As String, ByVal nodes As TreeNodeCollection, ByVal level As Integer)
        Dim Dirs() As String
        Try
            Dirs = Directory.GetDirectories(sDir)
            Dim sPath As String
            For Each sSubDir As String In Dirs
                If sSubDir(sDir.Length) = "\" Then
                    sPath = sSubDir.Substring(sDir.Length + 1)
                Else
                    sPath = sSubDir.Substring(sDir.Length)
                End If

                Dim node As New TreeNode()
                node.Text = sPath
                node.Name = sSubDir
                If level > 1 Then
                    AddTree(sSubDir, node.Nodes, level - 1)
                Else
                    Try
                        If Directory.GetDirectories(sSubDir).Length > 0 Then
                            node.Nodes.Add("temp") ' サブディレクトリがあるときダミーでノードを追加する
                            ' よって、この子フォルダを確認するときはNodesをClearすること
                        End If
                    Catch ex As Exception ' フォルダにアクセス出来ない場合スルー

                    End Try
                End If
                nodes.Add(node)
            Next
        Catch ex As Exception ' フォルダにアクセス出来ない場合スルー

        End Try
    End Sub

    ' パスの最後のフォルダ名を取得
    Private Function GetEndFolderName(ByVal strFolderPath As String) As String
        Dim strEndFolderName As String = ""
        If Not IsDBNull(strFolderPath) Then
            If strFolderPath.Length > 0 Then
                If Directory.Exists(strFolderPath) = False Then
                    Directory.CreateDirectory(strFolderPath)
                End If
                strFolderPath = PathEndAddSeparator(strFolderPath)
                Dim split As String() = Nothing
                Dim strDelim As String = "\"
                split = strFolderPath.Split(strDelim.ToCharArray())
                strEndFolderName = split(split.Length - 2)
            End If
        End If
        Return strEndFolderName
    End Function

    ' パスの最後に \ を追加
    Public Function PathEndAddSeparator(ByVal strPath As String) As String
        Dim strPathSp As String = strPath
        If Directory.Exists(strPath) = True Then
            If Not strPath(strPath.Length - 1) = Path.DirectorySeparatorChar Then
                strPathSp = strPath + Path.DirectorySeparatorChar
            End If
        End If
        Return strPathSp
    End Function

    ' 指定のパスに存在しないフォルダ名を取得
    Private Function GetNewFolderName(ByVal strNewFoldrName As String, ByVal iCount As Integer, ByVal strPath As String) As String
        Dim strNewFoder As String = ""
        If (iCount = 0) Then
            strNewFoder = strNewFoldrName
        Else
            strNewFoder = strNewFoldrName + iCount.ToString()
        End If
        System.GC.Collect()
        System.GC.WaitForPendingFinalizers()
        Try
            Dim di As DirectoryInfo = New DirectoryInfo(strPath)
            Dim dis As DirectoryInfo() = di.GetDirectories()
            If dis IsNot Nothing Then
                If dis.Length > 0 Then
                    For Each diData As DirectoryInfo In dis
                        If String.Compare(strNewFoder, diData.Name, True) = 0 Then
                            iCount = iCount + 1
                            strNewFoder = GetNewFolderName(strNewFoldrName, iCount, strPath)
                        End If
                    Next
                End If
            End If
        Catch ex As Exception

        End Try
        Return strNewFoder
    End Function

    ''' <summary>
    ''' 新しいフォルダを作成する
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function CreateNewFolder() As Boolean
        Dim bRet As Boolean = False
        If Me.SelectedNode Is Nothing Then
            Return False
        End If
        Dim strNewFolderPath As String = PathEndAddSeparator(Me.SelectedNode.Name)
        Dim strNewFolderName As String = GetNewFolderName("New Folder", 0, strNewFolderPath)
        If strNewFolderName IsNot "" Then
            strNewFolderPath = strNewFolderPath + strNewFolderName
            System.GC.Collect()
            System.GC.WaitForPendingFinalizers()
            Try
                Directory.CreateDirectory(strNewFolderPath)
                Dim tnChild As TreeNode = Me.SelectedNode.Nodes.Add(strNewFolderName)
                tnChild.Name = strNewFolderPath
                Me.SelectedNode = tnChild
                Me.Select()
                tnChild.BeginEdit()
                bRet = True
            Catch ex As Exception

            End Try
        End If

        Return bRet
    End Function

    ''' <summary>
    ''' フォルダの削除
    ''' </summary>
    ''' <param name="dir"></param>
    ''' <param name="bForce">bForceがTrueの時、空でないフォルダの削除確認メッセージを表示しない</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function DeleteFolder(ByVal dir As String, ByVal bForce As Boolean) As Boolean
        Dim bRet As Boolean = False
        Try
            Dim sFiles As String()
            If Not Directory.Exists(dir) Then
                MessageBox.Show(dir & " フォルダが存在しません。", My.Application.Info.Title, MessageBoxButtons.OK)
                Exit Try
            End If

            If bForce = False Then
                sFiles = Directory.GetFiles(dir)
                If sFiles.Length > 0 Then
                    If MessageBox.Show("フォルダ内にファイルが存在します。削除しますか？", My.Application.Info.Title, MessageBoxButtons.YesNo) = DialogResult.No Then
                        Exit Try
                    End If
                Else
                    sFiles = Directory.GetDirectories(dir)
                    If sFiles.Length > 0 Then
                        If MessageBox.Show("フォルダ内にサブフォルダが存在します。削除しますか？", My.Application.Info.Title, MessageBoxButtons.YesNo) = DialogResult.No Then
                            Exit Try
                        End If
                    End If
                End If
            End If

            Directory.Delete(dir, True)
            bRet = True
        Catch ex As Exception

        End Try
        Return bRet
    End Function

    ''' <summary>
    ''' 選択されているフォルダの削除
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function DeleteCurrentFolder() As Boolean
        Dim bRet As Boolean = False
        Dim bForce As Boolean = False

        If Me.SelectedNode Is Nothing Then
            Return False
        End If
        If Me.SelectedNode.Parent Is Nothing Then
            Return False
        End If

        System.GC.Collect()
        System.GC.WaitForPendingFinalizers()
        Try
            Dim tnParent As TreeNode = Me.SelectedNode.Parent
            Dim tnCurrent As TreeNode = Me.SelectedNode

            bRet = DeleteFolder(Me.SelectedNode.Name, bForce)
            If bRet Then
                Me.SelectedNode = tnParent
                Me.Nodes.Remove(tnCurrent)
            End If
        Catch ex As Exception

        End Try

        Return bRet
    End Function

    ''' <summary>
    ''' 選択されているフォルダのリネーム（開始）
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub RenameFolder()
        Dim tnCurrent As TreeNode = Me.SelectedNode

        tnCurrent.BeginEdit()
    End Sub

    ''' <summary>
    ''' 選択されているフォルダをクリップボードへコピー
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function ClipCopyFolder() As Boolean
        If Me.SelectedNode Is Nothing Then
            Return False
        End If

        Dim sc As Collections.Specialized.StringCollection = New Collections.Specialized.StringCollection
        Try
            sc.Add(Me.SelectedNode.Name) ' これだけでフォルダ以下がコピー
            System.Windows.Forms.Clipboard.SetFileDropList(sc)
        Catch ex As Exception

        End Try

        Return True
    End Function

    ''' <summary>
    ''' フォルダを中身ごとコピー
    ''' </summary>
    ''' <param name="sSource">コピー元のファイル・フォルダ</param>
    ''' <param name="sDest">コピー先のフォルダ</param>
    ''' <param name="bForce">上書き確認を出さないならTrue</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function CopyFolders(ByVal sSource As String, ByVal sDest As String, ByVal bForce As Boolean, ByVal bMove As Boolean) As Boolean
        Dim bRet As Boolean = True
        Dim sFolders As String()
        Dim sFiles As String()
        Dim sFolder As String
        Dim sFile As String
        Dim sNewFolder As String
        Dim sNewFile As String

        Try
            If sDest.Length >= sSource.Length Then
                If sDest.Substring(0, sSource.Length).CompareTo(sSource) = 0 Then
                    bRet = False
                    MessageBox.Show("コピーできません。コピー先にコピー元のフォルダが含まれています。", My.Application.Info.Title, MessageBoxButtons.OK)
                    Exit Try
                End If
            End If

            If Not Directory.Exists(sDest) Then
                Directory.CreateDirectory(sDest)
            End If
            If (File.GetAttributes(sSource) And FileAttributes.Directory) = FileAttributes.Directory Then
                sNewFolder = PathEndAddSeparator(sDest) + Path.GetFileName(sSource)
                If Not Directory.Exists(sNewFolder) Then
                    Directory.CreateDirectory(sNewFolder)
                End If

                sFiles = Directory.GetFiles(sSource)
                For Each sFile In sFiles
                    sNewFile = PathEndAddSeparator(sNewFolder) + Path.GetFileName(sFile)
                    Try
                        If Not bForce Then
                            If File.Exists(sNewFile) Then
                                If MessageBox.Show("ファイルが存在します。上書きしますか？", My.Application.Info.Title, MessageBoxButtons.YesNo) = DialogResult.No Then
                                    Continue For
                                End If
                            End If
                        End If
                        File.Copy(sFile, sNewFile, True)
                        If bMove Then
                            Try
                                File.Delete(sFile)
                            Catch ex As Exception
                                MessageBox.Show(sFile + "のコピーでエラーが発生しました（移動元が削除できません）", My.Application.Info.Title, MessageBoxButtons.OK)
                            End Try
                        End If
                    Catch ex As Exception
                        bRet = False
                        MessageBox.Show(sFile + "のコピーでエラーが発生しました。", My.Application.Info.Title, MessageBoxButtons.OK)
                    End Try
                Next

                sFolders = Directory.GetDirectories(sSource)
                For Each sFolder In sFolders
                    If Not CopyFolders(sFolder, sNewFolder, bForce, False) Then
                        bRet = False
                        Exit For
                    End If
                Next
            Else ' 最初がもうファイルだった場合の処理
                sNewFile = PathEndAddSeparator(sDest) + Path.GetFileName(sSource)
                If Not bForce Then
                    If File.Exists(sNewFile) Then
                        If MessageBox.Show("ファイルが存在します。上書きしますか？", My.Application.Info.Title, MessageBoxButtons.YesNo) = DialogResult.No Then
                            Return bRet
                        End If
                    End If
                End If
                File.Copy(sSource, sNewFile, True)
                If bMove Then
                    Try
                        File.Delete(sSource)
                    Catch ex As Exception
                        MessageBox.Show(sSource + "のコピーでエラーが発生しました（移動元が削除できません）", My.Application.Info.Title, MessageBoxButtons.OK)
                    End Try
                End If
            End If
        Catch ex As Exception
            bRet = False
            MessageBox.Show(sSource + "のコピーでエラーが発生しました。", My.Application.Info.Title, MessageBoxButtons.OK)
        End Try

        Return bRet
    End Function

    ''' <summary>
    ''' 選択されているフォルダへ貼り付け
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function PasteFolder() As Boolean
        If Me.SelectedNode Is Nothing Then
            Return False
        End If

        Dim sc As Collections.Specialized.StringCollection
        Dim bForce As Boolean = False

        If System.Windows.Forms.Clipboard.ContainsFileDropList() Then
            sc = System.Windows.Forms.Clipboard.GetFileDropList()
            For Each folderfile As String In sc
                System.GC.Collect()
                System.GC.WaitForPendingFinalizers()
                CopyFolders(folderfile, Me.SelectedNode.Name, bForce, False)
                Me.BeginUpdate()
                Me.SelectedNode.Nodes.Clear()
                AddTree(Me.SelectedNode.Name, Me.SelectedNode.Nodes, 2)
                Me.EndUpdate()
            Next
        End If

        Return True
    End Function

    ''' <summary>
    ''' ツリーのルートを取得する
    ''' </summary>
    ''' <param name="node"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function GetRootNode(ByVal node As TreeNode) As TreeNode
        Dim rNode As TreeNode = Nothing
        rNode = node
        If rNode Is Nothing Then
            Return rNode
        End If

        Do Until rNode.Parent Is Nothing
            rNode = rNode.Parent
        Loop

        Return rNode
    End Function

    ''' <summary>
    ''' ルートと、選択されているフォルダを中心とした再読み込み
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub ReloadFolders()
        If DesignMode Then
            Return
        End If

        Me.BeginUpdate()
        Try
            Dim node As TreeNode = Nothing
            Dim nodes As TreeNode()

            node = Me.SelectedNode
            If node Is Nothing Then
                Me.EndUpdate()
                Return
            End If
            Dim name As String = Me.SelectedNode.Name

            Dim rNode As TreeNode
            rNode = GetRootNode(node)
            Dim rName As String = ""
            If rNode Is Nothing Then
                rName = name
            Else
                rName = rNode.Name
            End If

            If Not name.StartsWith(rName) Then
                Me.EndUpdate()
                Return
            End If

            Dim strTrees As String()
            InitTree()

            If rName.CompareTo(name) = 0 Then
                nodes = Me.Nodes.Find(rName, False)
                rNode = nodes(0)
                nodes(0).Nodes.Clear()
                AddTree(nodes(0).Name, nodes(0).Nodes, 1)
                nodes(0).Expand()
                Me.SelectedNode = nodes(0)
                Me.EndUpdate()
                Return
            End If

            If rName(rName.Length - 1) = "\" Then
                name = name.Remove(0, rName.Length)
            Else
                name = name.Remove(0, rName.Length + 1)
            End If
            strTrees = name.Split("\")

            nodes = Me.Nodes.Find(rName, False)
            If nodes.Length = 0 Then
                Me.EndUpdate()
                Return
            End If

            rNode = nodes(0)
            nodes(0).Nodes.Clear()
            AddTree(nodes(0).Name, nodes(0).Nodes, 1)
            nodes(0).Expand()
            Dim sNname As String
            For Each strTree As String In strTrees
                sNname = nodes(0).Name
                If sNname(sNname.Length - 1) = "\" Then
                    nodes = nodes(0).Nodes.Find(nodes(0).Name + strTree, False)
                Else
                    nodes = nodes(0).Nodes.Find(nodes(0).Name + "\" + strTree, False)
                End If
                If nodes.Length = 0 Then
                    Exit For
                End If
                nodes(0).Nodes.Clear()
                AddTree(nodes(0).Name, nodes(0).Nodes, 1)
                nodes(0).Expand()
            Next
            Me.SelectedNode = nodes(0)
        Catch ex As Exception

        End Try

        Me.EndUpdate()
    End Sub
#End Region

#Region "プロパティ"
    ''' <summary>
    ''' マイピクチャフォルダを表示するか
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("FolderTreeView")> _
    <Description("マイピクチャフォルダを表示するか")> _
    Public Property ShowMyPicture() As Boolean
        Get
            Return prop_showMyPicthre
        End Get
        Set(ByVal value As Boolean)
            prop_showMyPicthre = value
        End Set
    End Property

    ''' <summary>
    ''' マイドキュメントフォルダを表示するか
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("FolderTreeView")> _
    <Description("マイドキュメントフォルダを表示するか")> _
    Public Property ShowMyDocument() As Boolean
        Get
            Return prop_showMyDocument
        End Get
        Set(ByVal value As Boolean)
            prop_showMyDocument = value
        End Set
    End Property

    ''' <summary>
    ''' リムーバブルドライブを表示するか
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("FolderTreeView")> _
    <Description("リムーバブルドライブを表示するか")> _
    Public Property ShowRemovable() As Boolean
        Get
            Return prop_showRemovable
        End Get
        Set(ByVal value As Boolean)
            prop_showRemovable = value
        End Set
    End Property

    ''' <summary>
    ''' ネットワークドライブを表示するか
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("FolderTreeView")> _
    <Description("ネットワークドライブを表示するか")> _
    Public Property ShowNetwork() As Boolean
        Get
            Return prop_showNetwork
        End Get
        Set(ByVal value As Boolean)
            prop_showNetwork = value
        End Set
    End Property

    ''' <summary>
    ''' フロッピードライブを表示するか
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("FolderTreeView")> _
    <Description("フロッピードライブを表示するか")> _
    Public Property ShowFloppy() As Boolean
        Get
            Return prop_showFloppy
        End Get
        Set(ByVal value As Boolean)
            prop_showFloppy = value
        End Set
    End Property
#End Region

End Class
