﻿Imports System.ComponentModel
Imports System.IO

Friend Class PhotoListUI
#Region "定数・メンバー変数・イベント"
    Private Const CtrlMask As Integer = 8 ' キーボード情報取得用
    Private Const ShiftMask As Integer = 4

    Private Const MAX_COLUMNS As Integer = 80 ' カラム最大値

    Private WithEvents m_imageDataList As ImageDataList
    Private sizeBaseImage As Size = New Size(1280 / 10, 960 / 10)

    Private m_iPrevSelectIndex As Integer = 0

    Private prop_bShowFileName As Boolean
    Private prop_bShowTitle As Boolean
    Private prop_bShowComment As Boolean

    Private prop_iHorzSpace As Integer = 2 ' プロパティ用変数
    Private prop_iVertSpace As Integer = 2
    Private prop_iColumns As Integer = 3
    Private prop_BaseFolder As String = Nothing

    Public Event DoubleClickEvent As MouseEventHandler
    Public Event UpdateEvent()
    Public Event SelItemChangeEvent(ByVal index As Integer, ByVal rc As Rectangle) ' 選択アイテムがキーボードで変更されたのを通知する

    Private m_iDragItem As Integer = -1 ' ドラッグ＆ドロップ用
    Private m_bDragNow As Boolean = False

    Private m_sizFont As SizeF = Nothing
#End Region

#Region "システム関数"
    Friend Sub New()
        MyBase.New()
        m_imageDataList = New ImageDataList
        m_imageDataList.ThumbSize = sizeBaseImage

        'This call is required by the Windows Form Designer. 
        InitializeComponent()

        Me.SetStyle(ControlStyles.UserPaint, True)
        Me.SetStyle(ControlStyles.DoubleBuffer, True)
        Me.SetStyle(ControlStyles.AllPaintingInWmPaint, True)
        Me.SetStyle(ControlStyles.SupportsTransparentBackColor, True)
        Me.BackColor = Color.White
    End Sub

    Protected Overrides Sub OnCreateControl()
        Me.Size = ImageAllSize(prop_iColumns, sizeBaseImage)
        MyBase.OnCreateControl()
    End Sub

    Public Sub Init()
        m_imageDataList.Init()
    End Sub

    Protected Overrides Sub OnPaint(ByVal pe As System.Windows.Forms.PaintEventArgs)
        MyBase.OnPaint(pe)

        If m_sizFont = Nothing Then ' 初回描画時に文字の高さを取得しておく
            m_sizFont = pe.Graphics.MeasureString("|", Me.Font)
            UpdateImageList()
        End If
        DrawImageList(pe.Graphics, prop_iColumns, ThumbSize)
    End Sub

    ''' <summary>
    ''' クリックイベント。選択項目設定などを行う
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub PhotoListUI_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Click
        Dim mouseEvent As MouseEventArgs = CType(e, MouseEventArgs)
        Dim itemi As Integer
        Dim i As Integer

        itemi = PointToItemIndex(mouseEvent.Location)

        If mouseEvent.Button = Windows.Forms.MouseButtons.Left Then

            If ModifierKeys = Keys.Control Then ' Ctrlが押されていれば、現在の項目のみ反転。範囲外ならなにもしない
                If itemi < 0 Then
                    Return
                End If
                m_imageDataList.Selected(itemi) = Not m_imageDataList.Selected(itemi)
                m_iPrevSelectIndex = itemi
                Invalidate()

            ElseIf ModifierKeys = Keys.Shift Then ' Shiftが押されていれば、一括選択。範囲外なら最後まで
                If itemi < 0 Then
                    itemi = m_imageDataList.Count - 1
                End If
                m_imageDataList.ClearSelect()
                If m_iPrevSelectIndex > itemi Then
                    For i = itemi To m_iPrevSelectIndex
                        m_imageDataList.Selected(i) = True
                    Next
                Else
                    For i = m_iPrevSelectIndex To itemi
                        m_imageDataList.Selected(i) = True
                    Next
                End If
                Invalidate()

            ElseIf ModifierKeys = Keys.Shift + Keys.Control Then ' Ctrl+Shiftなら、追加一括選択
                If itemi < 0 Then
                    itemi = m_imageDataList.Count - 1
                End If
                If m_iPrevSelectIndex > itemi Then
                    For i = itemi To m_iPrevSelectIndex
                        m_imageDataList.Selected(i) = True
                    Next
                Else
                    For i = m_iPrevSelectIndex To itemi
                        m_imageDataList.Selected(i) = True
                    Next
                End If
                Invalidate()

            Else ' キーが押されていなければ、単体選択
                If itemi < 0 Then
                    Return
                End If
                m_imageDataList.ClearSelect()
                m_iPrevSelectIndex = itemi
                m_imageDataList.Selected(itemi) = True
                Invalidate()
            End If

        End If
    End Sub

    ''' <summary>
    ''' ダブルクリックイベント。親に通知をする
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub PhotoListUI_DoubleClick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.DoubleClick
        Dim mouseEvent As MouseEventArgs = CType(e, MouseEventArgs)

        If mouseEvent.Button = Windows.Forms.MouseButtons.Left Then
            If ModifierKeys = Keys.Control Then ' Ctrl

            ElseIf ModifierKeys = Keys.Shift Then ' Shift

            Else ' 
                RaiseEvent DoubleClickEvent(sender, mouseEvent)
            End If

        End If

    End Sub

    ''' <summary>
    ''' キーボード処理。矢印キーはKeyDownで処理できないためPreviewKeyDownで行う
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub PhotoListUI_PreviewKeyDown(ByVal sender As System.Object, ByVal e As System.Windows.Forms.PreviewKeyDownEventArgs) Handles MyBase.PreviewKeyDown
        Select Case e.KeyCode
            Case Keys.Up
                If (Not e.Control) And (Not e.Shift) Then
                    SelKeyUp()
                End If
            Case Keys.Down
                If (Not e.Control) And (Not e.Shift) Then
                    SelKeyDown()
                End If
            Case Keys.Left
                If (Not e.Control) And (Not e.Shift) Then
                    SelKeyLeft()
                End If
            Case Keys.Right
                If (Not e.Control) And (Not e.Shift) Then
                    SelKeyRight()
                End If

        End Select
    End Sub

    ''' <summary>
    ''' キーボード処理。ショートカットキー
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub PhotoListUI_KeyDown(ByVal sender As System.Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles MyBase.KeyDown
        Select Case e.KeyCode
            Case Keys.F5
                Reflesh()

            Case Keys.D
                If e.Control Then
                    If DeleteFiles(False) = False Then
                        MessageBox.Show("いくつかのファイルを削除できませんでした。", My.Application.Info.Title, MessageBoxButtons.OK)
                    End If
                End If

            Case Keys.Delete
                If DeleteFiles(False) = False Then
                    MessageBox.Show("いくつかのファイルを削除できませんでした。", My.Application.Info.Title, MessageBoxButtons.OK)
                End If

            Case Keys.C
                If e.Control Then
                    ClipCopyFiles()
                End If

            Case Keys.V
                If e.Control Then
                    PasteFolder()
                End If

            Case Keys.A
                If e.Control Then
                    SelectAllImages()
                End If

            Case Keys.Enter
                Dim mea As MouseEventArgs = New MouseEventArgs(Windows.Forms.MouseButtons.Left, 1, 0, 0, 0)
                RaiseEvent DoubleClickEvent(sender, mea)

        End Select

    End Sub

    ''' <summary>
    ''' マウスボタン押下。右クリック時の処理と、ドラッグ＆ドロップの設定などを行う
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub PhotoListUI_MouseDown(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseDown
        Dim itemi As Integer
        itemi = PointToItemIndex(e.Location)
        m_iDragItem = -1

        If e.Button = Windows.Forms.MouseButtons.Right Then
            If itemi < 0 Then
                Return
            End If
            m_iPrevSelectIndex = itemi
            ' 既に選択されている項目上でのポップアップでは選択を維持し、選択されてない場所だったらこの項目のみを選択する
            If Not m_imageDataList.Selected(itemi) Then
                m_imageDataList.ClearSelect()
                m_imageDataList.Selected(itemi) = True
                Invalidate()
            End If
            ' 左ボタンで選択
        ElseIf e.Button = Windows.Forms.MouseButtons.Left Then
            If itemi >= 0 Then
                m_iDragItem = itemi
            End If
            ' CtrlやShiftなどが押されていなく、外でクリックされたら全選択解除
            If itemi < 0 And ModifierKeys = Keys.None Then
                m_imageDataList.ClearSelect()
                m_iPrevSelectIndex = 0
                Invalidate()
            End If
        End If
    End Sub

    ''' <summary>
    ''' マウス移動。ドラッグ＆ドロップ処理
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub PhotoListUI_MouseMove(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseMove
        Dim itemi As Integer
        itemi = PointToItemIndex(e.Location)

        If (SelectedImages.Count > 0) And (m_iDragItem >= 0) _
        And ((itemi < 0) Or (Not m_iDragItem = itemi)) Then
            m_bDragNow = True
            Dim paths(SelectedImages.Count - 1) As String
            For i As Integer = 0 To SelectedImages.Count - 1
                paths(i) = SelectedImages.Item(i).strName
            Next

            Dim data As New DataObject(DataFormats.FileDrop, paths)
            Dim dde As DragDropEffects

            dde = Me.DoDragDrop(data, DragDropEffects.All)
            If dde = DragDropEffects.Move Then
                RaiseEvent UpdateEvent()
            End If
            m_bDragNow = False
            m_iDragItem = -1
        End If
    End Sub

    Private Sub PhotoListUI_MouseUp(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles MyBase.MouseUp
        If Not m_bDragNow Then
            m_iDragItem = -1
        End If
    End Sub

    Private Sub PhotoListUI_MouseLeave(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.MouseLeave
        If Not m_bDragNow Then
            m_iDragItem = -1
        End If
    End Sub

    ''' <summary>
    ''' コントロール側にドラッグ＆ドロップされてきた時の処理
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub PhotoListUI_DragDrop(ByVal sender As System.Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles MyBase.DragDrop
        Dim filePathName As String() = e.Data.GetData(DataFormats.FileDrop, False)
        For Each strFile As String In filePathName
            If strFile.CompareTo(BaseFolder) = 0 Then

            ElseIf Not Path.GetDirectoryName(strFile).CompareTo(BaseFolder) = 0 Then
                ' 同じフォルダ上のファイルでなければ、コピーを行う
                If e.Effect = DragDropEffects.Move Then
                    CopyFolders(strFile, BaseFolder, False, True)
                ElseIf e.Effect = DragDropEffects.Copy Then
                    CopyFolders(strFile, BaseFolder, False, False)
                End If
            End If
        Next
        RaiseEvent UpdateEvent()
    End Sub

    ''' <summary>
    ''' コントロール側にドラッグ＆ドロップでカーソルが合わされている時の処理
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub PhotoListUI_DragEnter(ByVal sender As System.Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles MyBase.DragEnter
        ' ファイルドロップでは無いか、同じフォルダのファイルだった場合には処理は行わない
        If e.Data.GetDataPresent(DataFormats.FileDrop) Then
            Dim filePathName As String() = e.Data.GetData(DataFormats.FileDrop, False)

            Dim bCurFolder As Boolean = True
            For Each strFile As String In filePathName
                If strFile.CompareTo(BaseFolder) = 0 Then

                ElseIf Not Path.GetDirectoryName(strFile).CompareTo(BaseFolder) = 0 Then
                    bCurFolder = False
                End If
            Next

            If bCurFolder Then
                e.Effect = DragDropEffects.None
                Return
            End If

            If ((e.KeyState And CtrlMask) = CtrlMask) And ((e.AllowedEffect And DragDropEffects.Copy) = DragDropEffects.Copy) Then
                e.Effect = DragDropEffects.Copy
            ElseIf ((e.KeyState And ShiftMask) = ShiftMask) And ((e.AllowedEffect And DragDropEffects.Move) = DragDropEffects.Copy) Then
                e.Effect = DragDropEffects.Move
            Else
                e.Effect = DragDropEffects.Copy
            End If
        Else
            e.Effect = DragDropEffects.None
        End If
    End Sub

    ''' <summary>
    ''' イメージのリストが更新される時のイベント。表示更新などを行う
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub OnListUpdate(ByVal sender As Object, ByVal e As System.EventArgs) Handles m_imageDataList.SendEvent
        UpdateImageList()
        m_iPrevSelectIndex = 0
        Invalidate()
    End Sub
#End Region


#Region "ユーザー関数"
    ' 選択項目移動
    Private Sub SelKeyUp()
        If m_iPrevSelectIndex = 0 Then
            Return
        End If
        If m_iPrevSelectIndex - Columns() > 0 Then
            m_iPrevSelectIndex = m_iPrevSelectIndex - Columns()
        Else
            m_iPrevSelectIndex = 0
        End If
        m_imageDataList.ClearSelect()
        m_imageDataList.Selected(m_iPrevSelectIndex) = True
        Invalidate()
        RaiseEvent SelItemChangeEvent(m_iPrevSelectIndex, ItemIndexToRect(m_iPrevSelectIndex))
    End Sub
    ' 選択項目移動
    Private Sub SelKeyDown()
        If m_iPrevSelectIndex = (m_imageDataList.Count - 1) Then
            Return
        End If
        If m_iPrevSelectIndex + Columns() < (m_imageDataList.Count - 1) Then
            m_iPrevSelectIndex = m_iPrevSelectIndex + Columns()
        Else
            m_iPrevSelectIndex = m_imageDataList.Count - 1
        End If
        m_imageDataList.ClearSelect()
        m_imageDataList.Selected(m_iPrevSelectIndex) = True
        Invalidate()
        RaiseEvent SelItemChangeEvent(m_iPrevSelectIndex, ItemIndexToRect(m_iPrevSelectIndex))
    End Sub
    ' 選択項目移動
    Private Sub SelKeyRight()
        If m_iPrevSelectIndex = (m_imageDataList.Count - 1) Then
            Return
        End If
        If m_iPrevSelectIndex + 1 < (m_imageDataList.Count - 1) Then
            m_iPrevSelectIndex = m_iPrevSelectIndex + 1
        Else
            m_iPrevSelectIndex = m_imageDataList.Count - 1
        End If
        m_imageDataList.ClearSelect()
        m_imageDataList.Selected(m_iPrevSelectIndex) = True
        Invalidate()
        RaiseEvent SelItemChangeEvent(m_iPrevSelectIndex, ItemIndexToRect(m_iPrevSelectIndex))
    End Sub
    ' 選択項目移動
    Private Sub SelKeyLeft()
        If m_iPrevSelectIndex = 0 Then
            Return
        End If
        If m_iPrevSelectIndex - 1 > 0 Then
            m_iPrevSelectIndex = m_iPrevSelectIndex - 1
        Else
            m_iPrevSelectIndex = 0
        End If
        m_imageDataList.ClearSelect()
        m_imageDataList.Selected(m_iPrevSelectIndex) = True
        Invalidate()
        RaiseEvent SelItemChangeEvent(m_iPrevSelectIndex, ItemIndexToRect(m_iPrevSelectIndex))
    End Sub

    ' 全画像を選択状態にする
    Public Sub SelectAllImages()
        Dim i As Integer
        For i = 0 To m_imageDataList.Count - 1
            m_imageDataList.Selected(i) = True
        Next
        Invalidate()
    End Sub

    ' 必要な画面サイズを計算する
    Private Function ImageAllSize(ByVal myiColumn As Integer, ByVal sizeImage As Size) As Size
        Dim iRowNum As Integer = Int(m_imageDataList.Count / myiColumn)
        If (m_imageDataList.Count Mod myiColumn) <> 0 Then
            iRowNum += 1
        End If

        Dim h As Integer
        h = sizeImage.Height + prop_iVertSpace
        If Not m_sizFont = Nothing Then
            If prop_bShowFileName Then
                h += m_sizFont.Height
            End If
            If prop_bShowTitle Then
                h += m_sizFont.Height
            End If
            If prop_bShowComment Then
                h += m_sizFont.Height
            End If
        End If

        Return New Size(myiColumn * (sizeImage.Width + prop_iHorzSpace) _
            , iRowNum * h)
    End Function

    ' イメージを描画する
    Private Sub DrawImageList(ByVal g As Graphics, ByVal myiMaxColumn As Integer, ByVal sizeImage As Size)
        Dim iRowNum As Integer = Int(m_imageDataList.Count / myiMaxColumn)
        If (m_imageDataList.Count Mod myiMaxColumn) <> 0 Then
            iRowNum += 1
        End If
        g.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.HighQualityBicubic

        Dim rect As Rectangle
        Dim rect2 As Rectangle
        Dim iCount As Integer = 0
        Dim iTop As Integer = prop_iVertSpace / 2
        Dim iLeft As Integer = prop_iHorzSpace / 2
        Dim i As Integer = 0
        Dim imgdata As ImageData
        Dim dy As Integer = 0
        Dim pr1, pr2 As Double

        Dim bpen As Drawing.Pen = New Pen(Color.FromArgb(128, 128, 255))
        bpen.Width = 0
        bpen.DashStyle = Drawing2D.DashStyle.Dot
        Dim sFormat As StringFormat = New StringFormat
        sFormat.Alignment = StringAlignment.Center
        sFormat.LineAlignment = StringAlignment.Far


        For i = 0 To iRowNum - 1 Step i + 1
            Dim j As Integer = 0
            For j = 0 To myiMaxColumn - 1 Step j + 1
                If iCount < m_imageDataList.Count Then
                    imgdata = m_imageDataList(iCount)

                    ' 枠描画
                    dy = sizeImage.Height
                    If Not m_sizFont = Nothing Then
                        If prop_bShowFileName Then
                            dy += m_sizFont.Height
                        End If
                        If prop_bShowTitle Then
                            dy += m_sizFont.Height
                        End If
                        If prop_bShowComment Then
                            dy += m_sizFont.Height
                        End If
                    End If

                    rect2 = New Rectangle(iLeft - Int(prop_iHorzSpace / 2), iTop - Int(prop_iVertSpace / 2), sizeImage.Width + Int(prop_iHorzSpace / 2), dy + Int(prop_iVertSpace / 2))
                    If imgdata.bSelected Then
                        g.DrawRectangle(Pens.Cyan, rect2)
                        g.DrawRectangle(Pens.PaleTurquoise, rect2.Left + 1, rect2.Top + 1, rect2.Width - 2, rect2.Height - 2)
                        g.FillRectangle(Brushes.AliceBlue, rect2.Left + 2, rect2.Top + 2, rect2.Width - 3, rect2.Height - 3)
                    Else
                        g.DrawRectangle(Pens.LightGray, rect2)
                    End If

                    If imgdata.bLoaded Then
                        pr1 = sizeImage.Height / sizeImage.Width
                        pr2 = imgdata.img.Height / imgdata.img.Width
                        If pr1 < pr2 Then ' イメージが縦に長い場合
                            rect = New Rectangle(iLeft + (sizeImage.Width - sizeImage.Width * (pr1 / pr2)) / 2, iTop, sizeImage.Width * (pr1 / pr2), sizeImage.Height)
                        Else
                            rect = New Rectangle(iLeft, iTop + (sizeImage.Height - sizeImage.Height * (pr2 / pr1)) / 2, sizeImage.Width, sizeImage.Height * (pr2 / pr1))
                        End If

                        g.DrawImage(imgdata.img, rect)
                    Else
                        ' 未読み込み時
                        rect = New Rectangle(iLeft, iTop, sizeImage.Width, sizeImage.Height)
                        g.DrawRectangle(bpen, rect)
                        If Not prop_bShowFileName Then
                            g.DrawString(imgdata.strFile, Me.Font, Brushes.Black, rect, sFormat)
                        End If
                    End If

                    ' ファイル名表示
                    dy = sizeImage.Height

                    If Not m_sizFont = Nothing Then
                        If prop_bShowFileName Then
                            rect = New Rectangle(iLeft, iTop + dy, sizeImage.Width, m_sizFont.Height)
                            g.DrawString(imgdata.strFile, Me.Font, Brushes.Black, rect, sFormat)
                            dy += m_sizFont.Height
                        End If

                        ' タイトル表示
                        If prop_bShowTitle Then
                            rect = New Rectangle(iLeft, iTop + dy, sizeImage.Width, m_sizFont.Height)
                            If Not imgdata.strTitle Is Nothing Then
                                If imgdata.strTitle.Length > 0 Then
                                    g.DrawString(imgdata.strTitle, Me.Font, Brushes.Black, rect, sFormat)
                                End If
                            End If
                            dy += m_sizFont.Height
                        End If

                        ' コメント表示
                        If prop_bShowComment Then
                            rect = New Rectangle(iLeft, iTop + dy, sizeImage.Width, m_sizFont.Height)
                            If Not imgdata.strComment Is Nothing Then
                                If imgdata.strComment.Length > 0 Then
                                    g.DrawString(imgdata.strComment, Me.Font, Brushes.Black, rect, sFormat)
                                End If
                            End If
                            dy += m_sizFont.Height
                        End If
                    End If

                    iLeft += sizeImage.Width + prop_iHorzSpace
                    iCount += 1
                End If
            Next

            iLeft = prop_iHorzSpace / 2

            iTop += dy + prop_iVertSpace
        Next
    End Sub

    ' 座標からアイテムの番号を取得する
    Public Function PointToItemIndex(ByVal pt As Point) As Integer
        If IsDBNull(m_imageDataList) Then
            Return -1
        End If

        Dim x As Integer
        Dim y As Integer
        Dim i As Integer
        Dim h As Integer
        x = Int(pt.X / (ThumbSize.Width + prop_iHorzSpace))

        h = ThumbSize.Height + prop_iVertSpace
        If Not m_sizFont = Nothing Then
            If prop_bShowFileName Then
                h += m_sizFont.Height
            End If
            If prop_bShowTitle Then
                h += m_sizFont.Height
            End If
            If prop_bShowComment Then
                h += m_sizFont.Height
            End If
        End If
        y = Int(pt.Y / h)
        i = y * prop_iColumns + x
        If (i >= m_imageDataList.Count) Then
            Return -1
        End If

        Return i
    End Function

    ' アイテム番号から座標を取得する
    Public Function ItemIndexToRect(ByVal itemi As Integer) As Rectangle
        Dim x As Integer
        Dim y As Integer
        Dim x2 As Integer
        Dim y2 As Integer
        If IsDBNull(m_imageDataList) Then
            Return Nothing
        End If

        If itemi >= m_imageDataList.Count Or itemi < 0 Then
            Return Nothing
        End If

        x = itemi Mod prop_iColumns
        x2 = ThumbSize.Width + prop_iHorzSpace
        y = Int(itemi / prop_iColumns)
        y2 = ThumbSize.Height + prop_iVertSpace
        If Not m_sizFont = Nothing Then
            If prop_bShowFileName Then
                y2 += m_sizFont.Height
            End If
            If prop_bShowTitle Then
                y2 += m_sizFont.Height
            End If
            If prop_bShowComment Then
                y2 += m_sizFont.Height
            End If
        End If

        Return New Rectangle(x * x2, y * y2, x2, y2)
    End Function

    ' コントロールのサイズをリストに合わせる
    Public Sub UpdateImageList()
        Me.Size = ImageAllSize(prop_iColumns, ThumbSize)
    End Sub

    ' イメージのリストを全部クリアする
    Public Sub ClearImages()
        m_imageDataList.Clear()
    End Sub

    ' イメージを追加する
    Public Sub AddImageData(ByVal myImg As ImageData)
        m_imageDataList.Add(myImg)
    End Sub

    ' 選択解除する
    Public Sub ClearSelect()
        m_imageDataList.ClearSelect()
        Invalidate()
    End Sub

    ' 選択されているイメージを再読み込みする
    Public Sub ReloadSelected()
        Try
            For Each img As ImageData In SelectedImages
                m_imageDataList.ReloadFile(img.strName)
            Next
        Catch ex As Exception

        End Try
    End Sub

    ' 全てのイメージを再読み込みする
    Public Sub ReloadAll()
        Try
            For Each img As ImageData In AllImages
                m_imageDataList.ReloadFile(img.strName)
            Next
        Catch ex As Exception

        End Try
    End Sub

    ' 更新イベントを発行する
    Public Sub Reflesh()
        RaiseEvent UpdateEvent()
    End Sub

    ' 選択されているイメージのファイルのリストをクリップボードにコピーする
    Public Function ClipCopyFiles() As Boolean
        Dim bRet As Boolean = False

        Dim sc As Collections.Specialized.StringCollection = New Collections.Specialized.StringCollection
        Try
            For Each img As ImageData In SelectedImages
                sc.Add(img.strName)
            Next
            System.Windows.Forms.Clipboard.SetFileDropList(sc)
            bRet = True
        Catch ex As Exception

        End Try

        Return bRet
    End Function

    ' 選択されているファイルを削除する
    Public Function DeleteFiles(ByVal bForce As Boolean) As Boolean
        Dim bRet As Boolean = True

        Try
            If SelectedImages.Count > 0 Then
                If Not bForce Then
                    If MessageBox.Show("選択されているファイルを削除しますか？", My.Application.Info.Title, MessageBoxButtons.YesNo) = DialogResult.No Then
                        Exit Try
                    End If
                End If

                System.GC.Collect()
                System.GC.WaitForPendingFinalizers()
                For Each img As ImageData In SelectedImages
                    Try
                        File.Delete(img.strName)
                    Catch ex As Exception
                        bRet = False
                    End Try
                Next
            End If

            If bRet Then
                RaiseEvent UpdateEvent()
            End If
        Catch ex As Exception
            bRet = False
        End Try

        Return bRet
    End Function


    Private Function PathEndAddSeparator(ByVal strPath As String) As String
        Dim strPathSp As String = strPath
        If Directory.Exists(strPath) = True Then
            If Not strPath(strPath.Length - 1) = Path.DirectorySeparatorChar Then
                strPathSp = strPath + Path.DirectorySeparatorChar
            End If
        End If
        Return strPathSp
    End Function

    ''' <summary>
    ''' フォルダを中身ごとコピー
    ''' </summary>
    ''' <param name="sSource">コピー元のファイル・フォルダ</param>
    ''' <param name="sDest">コピー先のフォルダ</param>
    ''' <param name="bForce">上書き確認を出さないならTrue</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function CopyFolders(ByVal sSource As String, ByVal sDest As String, ByVal bForce As Boolean, ByVal bMove As Boolean) As Boolean
        Dim bRet As Boolean = True
        Dim sFolders As String()
        Dim sFiles As String()
        Dim sFolder As String
        Dim sFile As String
        Dim sNewFolder As String
        Dim sNewFile As String

        Try
            If sDest.Length >= sSource.Length Then
                If sDest.Substring(0, sSource.Length).CompareTo(sSource) = 0 Then
                    bRet = False
                    MessageBox.Show("コピーできません。コピー先にコピー元のフォルダが含まれています。", My.Application.Info.Title, MessageBoxButtons.OK)
                    Exit Try
                End If
            End If

            If Not Directory.Exists(sDest) Then
                Directory.CreateDirectory(sDest)
            End If
            If (File.GetAttributes(sSource) And FileAttributes.Directory) = FileAttributes.Directory Then
                sNewFolder = PathEndAddSeparator(sDest) + Path.GetFileName(sSource)
                If Not Directory.Exists(sNewFolder) Then
                    Directory.CreateDirectory(sNewFolder)
                End If

                sFiles = Directory.GetFiles(sSource)
                For Each sFile In sFiles
                    sNewFile = PathEndAddSeparator(sNewFolder) + Path.GetFileName(sFile)
                    Try
                        If Not bForce Then
                            If File.Exists(sNewFile) Then
                                If MessageBox.Show("ファイルが存在します。上書きしますか？", My.Application.Info.Title, MessageBoxButtons.YesNo) = DialogResult.No Then
                                    Continue For
                                End If
                            End If
                        End If
                        File.Copy(sFile, sNewFile, True)
                        If bMove Then
                            Try
                                File.Delete(sFile)
                            Catch ex As Exception
                                MessageBox.Show(sFile + "のコピーでエラーが発生しました（移動元が削除できません）", My.Application.Info.Title, MessageBoxButtons.OK)
                            End Try
                        End If
                    Catch ex As Exception
                        bRet = False
                        MessageBox.Show(sFile + "のコピーでエラーが発生しました。", My.Application.Info.Title, MessageBoxButtons.OK)
                    End Try
                Next

                sFolders = Directory.GetDirectories(sSource)
                For Each sFolder In sFolders
                    If Not CopyFolders(sFolder, sNewFolder, bForce, False) Then
                        bRet = False
                        Exit For
                    End If
                Next
            Else ' 最初がもうファイルだった場合の処理
                sNewFile = PathEndAddSeparator(sDest) + Path.GetFileName(sSource)
                If Not bForce Then
                    If File.Exists(sNewFile) Then
                        If MessageBox.Show("ファイルが存在します。上書きしますか？", My.Application.Info.Title, MessageBoxButtons.YesNo) = DialogResult.No Then
                            Return bRet
                        End If
                    End If
                End If
                File.Copy(sSource, sNewFile, True)
                If bMove Then
                    Try
                        File.Delete(sSource)
                    Catch ex As Exception
                        MessageBox.Show(sSource + "のコピーでエラーが発生しました（移動元が削除できません）", My.Application.Info.Title, MessageBoxButtons.OK)
                    End Try
                End If
            End If
        Catch ex As Exception
            bRet = False
            MessageBox.Show(sSource + "のコピーでエラーが発生しました。", My.Application.Info.Title, MessageBoxButtons.OK)
        End Try

        Return bRet
    End Function

    ' クリップボードのファイルリストを現フォルダにコピー
    Public Function PasteFolder() As Boolean
        Dim sc As Collections.Specialized.StringCollection
        Dim bForce As Boolean = False

        If (BaseFolder IsNot Nothing) And _
        System.Windows.Forms.Clipboard.ContainsFileDropList() Then
            sc = System.Windows.Forms.Clipboard.GetFileDropList()
            For Each folderfile As String In sc
                System.GC.Collect()
                System.GC.WaitForPendingFinalizers()
                CopyFolders(folderfile, BaseFolder, bForce, False)
            Next
        End If

        RaiseEvent UpdateEvent()
        Return True
    End Function

#End Region

#Region "プロパティ"

    ''' <summary>
    ''' カラム数の最大値（読み取り専用）
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("PhotoListUI")> _
    Public ReadOnly Property MaxCol()
        Get
            Return MAX_COLUMNS
        End Get
    End Property


    ''' <summary>
    ''' リストの横スペース
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("PhotoListUI")> _
    <DefaultValue(2)> _
    Public Property HorzSpace() As Integer
        Get
            Return prop_iHorzSpace
        End Get
        Set(ByVal value As Integer)
            If Not prop_iHorzSpace = value Then
                prop_iHorzSpace = value
                UpdateImageList()
                Invalidate()
            End If
        End Set
    End Property

    ''' <summary>
    ''' リストの縦スペース
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("PhotoListUI")> _
    <DefaultValue(2)> _
    Public Property VertSpace() As Integer
        Get
            Return prop_iVertSpace
        End Get
        Set(ByVal value As Integer)

            If Not prop_iVertSpace = value Then
                prop_iVertSpace = value
                UpdateImageList()
                Invalidate()
            End If
        End Set
    End Property

    ''' <summary>
    ''' 横カラム数
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("PhotoListUI")> _
    <DefaultValue(5)> _
    Public Property Columns() As Integer
        Get
            Return prop_iColumns
        End Get
        Set(ByVal value As Integer)
            If value > MAX_COLUMNS Then
                value = MAX_COLUMNS
            End If
            If value <= 0 Then
                value = 1
            End If
            If Not prop_iColumns = value Then
                prop_iColumns = value
                UpdateImageList()
                Invalidate()
            End If
        End Set
    End Property

    ''' <summary>
    ''' ファイル名を表示するか
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("PhotoListUI")> _
    <DefaultValue(True)> _
    Public Property ShowFileName() As Boolean
        Get
            Return prop_bShowFileName
        End Get
        Set(ByVal value As Boolean)
            If Not prop_bShowFileName = value Then
                prop_bShowFileName = value
                UpdateImageList()
                Invalidate()
            End If
        End Set
    End Property

    ''' <summary>
    ''' タイトルを表示するか
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("PhotoListUI")> _
    <DefaultValue(False)> _
    Public Property ShowTitle() As Boolean
        Get
            Return prop_bShowTitle
        End Get
        Set(ByVal value As Boolean)
            If Not prop_bShowTitle = value Then
                prop_bShowTitle = value
                UpdateImageList()
                Invalidate()
            End If
        End Set
    End Property

    ''' <summary>
    ''' コメントを表示するか
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("PhotoListUI")> _
    <DefaultValue(False)> _
    Public Property ShowComment() As Boolean
        Get
            Return prop_bShowComment
        End Get
        Set(ByVal value As Boolean)
            If Not prop_bShowComment = value Then
                prop_bShowComment = value
                UpdateImageList()
                Invalidate()
            End If
        End Set
    End Property

    ' サムネイルサイズの設定・取得
    <Category("PhotoListUI")> _
    Public Property ThumbSize() As Size
        Get
            If m_imageDataList Is Nothing Then
                Return sizeBaseImage
            End If
            Return m_imageDataList.ThumbSize
        End Get
        Set(ByVal value As Size)
            If m_imageDataList Is Nothing Then
                Return
            End If
            m_imageDataList.ThumbSize = value
            UpdateImageList()
            Invalidate()
        End Set
    End Property

    ' 選択されているImageDataのリストを返す
    <Category("PhotoListUI")> _
    Public ReadOnly Property SelectedImages() As ImageDataList
        Get
            Dim imgData As ImageDataList = Nothing

            If m_imageDataList Is Nothing Then
                Return imgData
            End If

            imgData = New ImageDataList
            For Each imgd As ImageData In m_imageDataList
                If imgd.bSelected Then
                    imgData.Add(imgd)
                End If
            Next

            Return imgData
        End Get
    End Property

    ' 全ImageDataのリストを返す
    <Category("PhotoListUI")> _
    Public ReadOnly Property AllImages() As ImageDataList
        Get
            Dim imgData As ImageDataList = Nothing

            If m_imageDataList Is Nothing Then
                Return imgData
            End If

            imgData = New ImageDataList
            For Each imgd As ImageData In m_imageDataList
                imgData.Add(imgd)
            Next

            Return imgData
        End Get
    End Property

    ''' <summary>
    ''' 開いているフォルダ
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Category("PhotoListUI")> _
    Public Property BaseFolder() As String
        Get
            Return prop_BaseFolder
        End Get
        Set(ByVal value As String)
            prop_BaseFolder = value
        End Set
    End Property
#End Region

End Class
